import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;

/**
 * A panel w/ two JLists (1 for directory, 1 for files in that directory)
 * for opening an image (a la GQView).
 */
class FilePanel extends Box
                implements ActionListener,
                           MouseListener,
                           ListSelectionListener  {

	private JComboBox currDirField;
	private JList fileList, dirList;
	private DefaultListModel fileModel, dirModel;
	private JScrollPane fileSP, dirSP, previewSP;
	private JButton open;
	private ImagePreview imagePreview;

	private ImageViewer parent;
	private Properties properties;

	private boolean createThumbsInBackground;

	private String slash;

	// constants
	static final String currDirS = "currDir";
	static final String openS = "open";
	static final String dotDotS = "..";

	public FilePanel(ImageViewer _parent, Properties prop) {

		super(BoxLayout.Y_AXIS);

		parent = _parent;
		properties = prop;

		slash = System.getProperty("file.separator");

		imagePreview = new ImagePreview(parent);
		String temp = properties.getProperty(PropertiesConstants.cacheThumbsS);
		boolean enable = Boolean.valueOf(temp).booleanValue();
		imagePreview.enableThumbnailCaching(enable);

		String dir = properties.getProperty(PropertiesConstants.imageDirS);
		setupCurrDirField(dir);

		temp = properties.getProperty(PropertiesConstants.createThumbsInBackgroundS);
		createThumbsInBackground = Boolean.valueOf(temp).booleanValue();

		fileModel = new DefaultListModel();
        	dirModel = new DefaultListModel();

		buildGUI();

                readDir(dir);

	}

	private void buildGUI() {

		fileList = new JList(fileModel);
		dirList = new JList(dirModel);

		String s = properties.getProperty(PropertiesConstants.previewImagesS);
		boolean preview = Boolean.valueOf(s).booleanValue();
		if (preview)
		   fileList.addListSelectionListener(this);

		dirList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

		setListProps(fileList);
		setListProps(dirList);

		fileSP = new JScrollPane(fileList);
		dirSP = new JScrollPane(dirList);

                open = new JButton("Open");
		open.setMnemonic(KeyEvent.VK_O);
		open.addActionListener(this);
		open.setActionCommand(openS);

		setOpenAsDefaultButton();

		setSizes();

		add(imagePreview);//previewSP);
                add(currDirField);
                add(dirSP);
                add(fileSP);
                add(open);

	}

	private void setupCurrDirField(String dir) {

		String[] currDirData;
		String imageDir = dir;
	        if ( !imageDir.endsWith(slash) )
	           imageDir += slash;

		File[] roots = File.listRoots();
		currDirData = new String[roots.length+2];
		currDirData[0] = imageDir;
		currDirData[1] = System.getProperty("user.home")+slash;
		for (int i = 0; i < roots.length; i++)
                    currDirData[i+2] = roots[i].getAbsolutePath();
                currDirField = new JComboBox(currDirData);
		currDirField.getEditor().setItem(imageDir);
		currDirField.setEditable(true);
		currDirField.addActionListener(this);
		currDirField.setActionCommand(currDirS);

	}

	private void setListProps(JList list) {

		list.setVisibleRowCount(10);
                list.setPrototypeCellValue("1234567890123456789");
                list.addMouseListener(this);

	}

	public void setOpenAsDefaultButton() {

		// set open as the default button
		parent.getRootPane().setDefaultButton(open);

	}

	private void setSizes() {

		Dimension d0 = new Dimension(170, 25);
		Dimension d0a = new Dimension(200, 300);
                Dimension d1 = new Dimension(150, 150);
		Dimension d1a = new Dimension(170, 180);
		Dimension d3 = new Dimension(150, 100);
                Dimension d3a = new Dimension(170, 130);
                imagePreview.setAlignmentX(Component.LEFT_ALIGNMENT);
		//previewSP.setAlignmentX(Component.LEFT_ALIGNMENT);
		//previewSP.setPreferredSize(d3);
		//previewSP.setMaximumSize(d0a);
                currDirField.setAlignmentX(Component.LEFT_ALIGNMENT);
                currDirField.setMinimumSize(d0);
		currDirField.setPreferredSize(d0);
                currDirField.setMaximumSize( new Dimension(200, 30) );
                dirSP.setAlignmentX(Component.LEFT_ALIGNMENT);
                dirSP.setMinimumSize(d3);
                dirSP.setPreferredSize(d3a);
                dirSP.setMaximumSize(d0a);
                fileSP.setAlignmentX(Component.LEFT_ALIGNMENT);
		fileSP.setMinimumSize(d1);
		fileSP.setPreferredSize(d1a);
		fileSP.setMaximumSize(d0a);
                open.setAlignmentX(Component.LEFT_ALIGNMENT);

	}

	private void readDir(String dir) {

		Vector fileElems = new Vector();
		Vector dirElems = new Vector();

		File file = new File(dir);

        	File[] files = file.listFiles();

        	if (files == null)
        	   return;

		getFilesAndDirs(files, fileElems, dirElems);

		// sort files and directories alphabetically
        	Collections.sort(fileElems, String.CASE_INSENSITIVE_ORDER);
        	Collections.sort(dirElems, String.CASE_INSENSITIVE_ORDER);

		fileModel.clear();
        	dirModel.clear();

		dirModel.addElement(dotDotS);
		int size = dirElems.size();
		for (int i = 0; i < size; i++)
		    dirModel.addElement( dirElems.get(i) );
		size = fileElems.size();
		for (int i = 0; i < size; i++)
                    fileModel.addElement( fileElems.get(i) );

	}

	private void getFilesAndDirs(File[] files, Vector fileElems, Vector dirElems) {

		for (int i = 0; i < files.length; i++) {

        	    if ( files[i].isFile() ) {
                       // use file filters (if set in prefs)
                       String prop = properties.getProperty(PropertiesConstants.useFileFiltersS);
                       boolean useFilters = Boolean.valueOf(prop).booleanValue();
                       if (useFilters) {
                          String ext = Utils.getExtension(files[i]);
                          if (ext != null) {
                             int index = Utils.allTypes.indexOf(ext);
                             if ( index > -1 )
                                fileElems.addElement( files[i].getName() ); // add to list model for files
                          }
                       }
                       else
                          fileElems.addElement( files[i].getName() ); // add to list model for files
                    }
                    else
                       dirElems.addElement( files[i].getName() ); // add to list model for directories

                }

        }

        /**
         * Enables image previews in the FilePanel
         * if enable is true.
         * If enable is false, image previews are
         * disabled.
         */
        public void enablePreviews(boolean enable) {

        	if (enable)
                   fileList.addListSelectionListener(this);
        	else
        	   fileList.removeListSelectionListener(this);

        }

        public void enableBackgroundThumbsCreation(boolean enable) {

        	createThumbsInBackground = enable;

        }

        public ImagePreview getImagePreviewer() {

		return imagePreview;

        }

        public void setCurrDirItem(String item) {

        	currDirField.getEditor().setItem(item);
        	currDirAction(item);

        }

        public void valueChanged(ListSelectionEvent e) {

        	previewImage(); // selected item in the files list
        	                // has changed, update the image preview

        }

	public void actionPerformed(ActionEvent e) {

		String actionCommand = e.getActionCommand();

		if ( actionCommand.equals(currDirS) )
		   currDirAction(null);
		else if ( actionCommand.equals(openS) )
		   openAction();

	}

	private void currDirAction(String currDir) {

		if (currDir == null) {

		   if ( currDirField.getSelectedIndex() < 0 )
		      currDir = (String)currDirField.getEditor().getItem();
		   else {
		      currDir = (String)currDirField.getSelectedItem();
                      currDirField.getEditor().setItem(currDir);
		   }

		}

		if ( !currDir.endsWith(slash) ) {
		   currDir += slash;
		   currDirField.getEditor().setItem(currDir);
		}

		readDir(currDir);

	}

	/**
	 * Opens the selected file(s).
	 */
	private void openAction() {

                Object[] objs = fileList.getSelectedValues();
		String[] files = new String[objs.length];
		String currDir = (String)currDirField.getEditor().getItem();

		for (int i = 0; i < objs.length; i++) {
                    files[i] = currDir + (String)objs[i];
                    parent.showImageAI( new File(files[i]) );
                }

	}

	// empty mouse methods must be here so that this
        //  class meets the mouse listener interface

	public void mouseEntered(MouseEvent me) {}

        public void mouseExited(MouseEvent me) {}

        public void mousePressed(MouseEvent me) {}

        public void mouseReleased(MouseEvent me) {}

        public void mouseClicked(MouseEvent me) {

        	JList list = (JList)me.getSource();
        	int clickCt = me.getClickCount();

        	if (clickCt == 2) {  // double click

                   if (list == fileList)
                      openAction(); // open the clicked on file


        	}
        	else if (clickCt == 1) { // single click

        	   if (list == dirList)
        	      dirListClicked(); // update file list for the clicked on directory

        	}

        }

        /**
         * Updates the file list for the directory name that was
         * just clicked on.
         */
        private void dirListClicked() {

        	String dir = (String)dirList.getSelectedValue();

                String prevDir = (String)currDirField.getEditor().getItem();

                String fullDir;
                // handle .. directory specially
        	if ( dir.equals(dotDotS) ) {
                   fullDir = ( new File(prevDir) ).getParent();
                   if ( !fullDir.endsWith(slash) )
                      fullDir += slash;
                }
                else {
		   fullDir = prevDir + dir;
		   if ( !fullDir.endsWith(slash) )
		      fullDir += slash;
                }

		if (fullDir != null) {
        	   readDir(fullDir);
        	   currDirField.getEditor().setItem(fullDir);
                }

        }

        private void previewImage() {

        	String filename = (String)currDirField.getEditor().getItem() +
        	        	   (String)fileList.getSelectedValue();
        	if (filename != null)
        	   imagePreview.updateImage(filename);

        }

}
