import javax.swing.*;
import java.beans.*;
import java.awt.*;
import java.io.*;
import java.util.Vector;

public class ImagePreview extends JComponent
                          implements PropertyChangeListener {

    /**
     * The maximum size, in bytes, to not create a
     * saved-to-disk thumbnail for an image preview.
     */
    public static final long MAX_FILE_SIZE = 6000;

    /**
     * The name of the directory to store thumbnails in
     * (relative to the location of the full-sized image).
     */
    public static String THUMBNAIL_DIR = ".jimageviewpics";

    private ImageIcon thumbnail = null;
    private File file = null;
    private int width;
    private ImageViewer parent;
    private ImageFilter imageFilter;
    private boolean cacheThumbnails;
    private boolean createThumbsInBackground;
    private boolean imageFilterCreated = false;

    public ImagePreview(ImageViewer _parent) {
        parent = _parent;
        setMinimumSize(new Dimension(150, 150));
        setPreferredSize(new Dimension(150, 160));
        setMaximumSize(new Dimension(200, 300));
        width = 140;
    }

    public ImagePreview(JFileChooser fc, ImageViewer _parent) {
        parent = _parent;
        setPreferredSize(new Dimension(100, 50));
        fc.addPropertyChangeListener(this);
        width = 90;
    }

    public void enableThumbnailCaching(boolean enable) {

    	cacheThumbnails = enable;

    }

    /**
     * Call only for JFileChooser, not for FilePanel
     */
    public void enableBackgroundThumbsCreation(boolean enable) {

	createThumbsInBackground = enable;

    }

    /**
     * Called by the JList of files in FilePanel
     * to load a new image preview.
     */
    public void updateImage(String filename) {

	file = new File(filename);
    	if ( isShowing() ) {
    	   loadImage();
    	   repaint();
        }

    }

    public void loadImage() {

        if (file == null)
            return;

        ImageIcon tmpIcon = new ImageIcon(file.getPath());
        if (tmpIcon.getIconWidth() > width)
           thumbnail = new ImageIcon( tmpIcon.getImage().getScaledInstance(width,
                                          -1, Image.SCALE_DEFAULT) );
        else
           thumbnail = tmpIcon;

    }

    public void propertyChange(PropertyChangeEvent e) {
        String prop = e.getPropertyName();
        if (prop.equals(JFileChooser.SELECTED_FILE_CHANGED_PROPERTY)) {
            file = (File) e.getNewValue();
            if (isShowing()) {
                loadImage();
                repaint();
            }
        }

    }

    public void paintComponent(Graphics g) {
        if (thumbnail == null) {
            loadImage();
        }
        if (thumbnail != null) {
            int x = getWidth()/2 - thumbnail.getIconWidth()/2;
            int y = getHeight()/2 - thumbnail.getIconHeight()/2;

            if (y < 0) {
                y = 0;
            }

            if (x < 5) {
                x = 5;
            }
            thumbnail.paintIcon(this, g, x, y);
        }
    }
}
