import java.io.*;
import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.awt.image.renderable.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.filechooser.*;
import java.util.*;
import java.beans.*;
import java.net.URL;

/*
   Copyright 2001 Joseph McMurry

   Released under the GPL (version 2)
    (see GPL.txt for more info)
*/

/**
 * @author Joseph McMurry <BR>
 * 2/5/2001              <BR>
 * JImageView application <BR>
 * Allows viewing of GIF, JPEG, and PNG images <BR>
 * Works without the Java Advanced Imaging API <BR>
 * @version 0.3        <BR>
 * @since JDK 1.2
 */
public class ImageViewer extends JFrame
                         implements ActionListener,
                                      MouseListener,
                                        PropertyChangeListener {

	private JDesktopPane desktopPane;
	private ImageInternalFrame frame;
	private JFileChooser fileChooser;
	private FileDialog fileDialog;
	private JComboBox scaleChoice;
	private PrefsDialog prefsDialog;
	private ScaleDialog scaleDialog;
	private JPopupMenu popup;
	private JToolBar toolBar;
	private JPanel contentPane;
	private JSplitPane splitPane;
	private FilePanel filePanel;
	private JMenuItem ocFilePanel;

	private boolean fileChooserCreated;
	private boolean prefsDialogCreated;
	private boolean scaleDialogCreated;
	private boolean filePanelCreated;
	private boolean useSwingFileChooser;
	private boolean previewImages;
	private boolean useJavaLF;
	private boolean useFileFilters;
	private boolean useFileTypeIcons;
	private boolean useFilePanel;

	private OpenFrameList openFrameList;
	private int numFramesCreated; // the number of internal frames created so far
	private int openFrameCount; // the number of open internal frames
	private String[] scaleSizes;
        private Properties properties;

	public static String slashS;


	private ImageViewer(String[] args) {

		initBaseVars();

		slashS = System.getProperty("file.separator");

		openFrameList = new OpenFrameList();

		loadProperties();

		buildGUI();

                // open the file named on the command line, if any
                if ( args.length > 0 )
                   showImageAI( new File(args[0]) );

	}


	private void initBaseVars() {

		fileChooserCreated = false;
		prefsDialogCreated = false;
		scaleDialogCreated = false;
		filePanelCreated = false;

		numFramesCreated = 0;
		openFrameCount = 0;

	}

	private void buildGUI() {

		setLF();

                createMenus();
                createPopupMenu();

                contentPane = new JPanel();
                contentPane.setLayout( new BorderLayout() );

                createToolBar();
                layoutToolBar();

                desktopPane = new JDesktopPane();

                if (useFilePanel) {
                   filePanel = new FilePanel(this, properties);
                   filePanelCreated = true;
                   setupSplitPane();
                }
                else
                   contentPane.add( desktopPane, BorderLayout.CENTER );

		setContentPane(contentPane);

                String prop = properties.getProperty(PropertiesConstants.dragModeS);
                if ( !(prop).equals(PropertiesConstants.liveS) ) { // if dragMode != live
                   // make dragging internal frames faster
                   desktopPane.putClientProperty("JDesktopPane.dragMode", PropertiesConstants.outlineS);
                }

                URL iconURL = ClassLoader.getSystemResource("icons/program.gif");
		Image img = Toolkit.getDefaultToolkit().getImage(iconURL);
                setIconImage(img);

		openMainWindow();

	}

	private void setLF() {

		try {
		     /*if (useJavaLF)
                        UIManager.setLookAndFeel( UIManager.getCrossPlatformLookAndFeelClassName() );*/
                     if (!useJavaLF) {
		        String os = System.getProperty("os.name");
			int index = os.indexOf("Linux");
			if (index > -1) // use Motif L&F on Linux
		           UIManager.setLookAndFeel("com.sun.java.swing.plaf.motif.MotifLookAndFeel");
                        else
			   UIManager.setLookAndFeel( UIManager.getSystemLookAndFeelClassName() );
		     }
                }
                catch (Exception e) {}

	}

	private void layoutToolBar() {

		String toolBarPos = properties.getProperty(PropertiesConstants.toolBarPosS);

                String toolBarLayout;
                if ( toolBarPos.equals(PropertiesConstants.topS) )
                   toolBarLayout = BorderLayout.NORTH;
                else if ( toolBarPos.equals(PropertiesConstants.bottomS) )
                   toolBarLayout = BorderLayout.SOUTH;
                else if ( toolBarPos.equals(PropertiesConstants.leftS) )
                   toolBarLayout = BorderLayout.WEST;
                else // if (right)
                   toolBarLayout = BorderLayout.EAST;
                contentPane.add( toolBar, toolBarLayout );

	}

	private void setupSplitPane() {

        	splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
                                               filePanel, desktopPane);
                splitPane.setOneTouchExpandable(true);
                splitPane.setDividerLocation(170);
                splitPane.addPropertyChangeListener(JSplitPane.DIVIDER_LOCATION_PROPERTY, this);
                contentPane.add(splitPane, BorderLayout.CENTER );

	}

	private void openMainWindow() {

		setTitle("JImageView");

	        Dimension d = Toolkit.getDefaultToolkit().getScreenSize();

	        String prop = properties.getProperty(PropertiesConstants.fullScreenS);
	        boolean fullScreen = Boolean.valueOf(prop).booleanValue();
		int width, height;

	        if (!fullScreen) {
                   width = Integer.parseInt( properties.getProperty(PropertiesConstants.widthS) );
	           height = Integer.parseInt( properties.getProperty(PropertiesConstants.heightS) );
	        }
	        else { // fullScreen
	           width = (int)d.getWidth();
	           height = (int)d.getHeight();
	        }
	        pack();
	        setSize(width, height);

	        // center the imageViewer window (JFrame) on the screen
	        Rectangle rect = new Rectangle(new Point(0,0), d);
                setLocation( (rect.x + ((rect.width - width)/2)),
		               (rect.y + ((rect.height - height)/2)) );

		setVisible(true);

	}

        /**
         * Reads the configuration file
         * into the Properties object.
         */
	private void loadProperties() {

     		properties = new Properties();
     		FileInputStream in = null;
     		BufferedInputStream bis = null;
     		String dir = System.getProperty(IVConstants.userHomeS);
     		String configFileWPath = dir+slashS+IVConstants.configFilename;
     		try {
     		     in = new FileInputStream(configFileWPath);
                     bis = new BufferedInputStream(in);
                     properties.load(bis);
                     String prop = properties.getProperty(PropertiesConstants.useSwingFileChooserS);
	             useSwingFileChooser = Boolean.valueOf(prop).booleanValue();
	             prop = properties.getProperty(PropertiesConstants.previewImagesS);
	             previewImages = Boolean.valueOf(prop).booleanValue();
	             prop = properties.getProperty(PropertiesConstants.useJavaLFS);
	             useJavaLF = Boolean.valueOf(prop).booleanValue();
	             prop = properties.getProperty(PropertiesConstants.useFileFiltersS);
	             useFileFilters = Boolean.valueOf(prop).booleanValue();
	             prop = properties.getProperty(PropertiesConstants.useFileTypeIconsS);
	             useFileTypeIcons = Boolean.valueOf(prop).booleanValue();
	             prop = properties.getProperty(PropertiesConstants.useFilePanelS);
	             useFilePanel = Boolean.valueOf(prop).booleanValue();
	        }
                catch (Exception e) {
                     // assume defaults for config file since one does not exit
                     System.out.println("Configuration file does not exist");
                     setProps();
                     // write default config file to disk
                     try {
                          FileOutputStream out = new FileOutputStream(configFileWPath);
             		  BufferedOutputStream bos = new BufferedOutputStream(out);
                          properties.store(bos, null);
                          bos.close();
                          out.close();
                     }
                     catch (IOException ioe) {
                     	  System.err.println("Properties file could not be saved");
                     }
                }

        }

        private void setProps() {

                     useSwingFileChooser = true;
                     previewImages = true;
                     useJavaLF = true;
                     useFileFilters = true;
                     useFileTypeIcons = true;
                     useFilePanel = true;
        	     properties.setProperty(PropertiesConstants.useSwingFileChooserS, IVConstants.trueS);
                     properties.setProperty(PropertiesConstants.dragModeS, PropertiesConstants.outlineS);
                     properties.setProperty( PropertiesConstants.imageDirS,
                                              System.getProperty(IVConstants.userHomeS) );
                     properties.setProperty(PropertiesConstants.widthS, "730");
                     properties.setProperty(PropertiesConstants.heightS, "530");
                     properties.setProperty(PropertiesConstants.fullScreenS, IVConstants.falseS);
                     properties.setProperty(PropertiesConstants.toolBarPosS, PropertiesConstants.topS);
                     properties.setProperty(PropertiesConstants.previewImagesS, IVConstants.trueS);
                     properties.setProperty(PropertiesConstants.useJavaLFS, IVConstants.trueS);
                     properties.setProperty(PropertiesConstants.useFileFiltersS, IVConstants.trueS);
                     properties.setProperty(PropertiesConstants.useFileTypeIconsS, IVConstants.trueS);
		     properties.setProperty(PropertiesConstants.resizeOnScaleS, IVConstants.falseS);
                     properties.setProperty(PropertiesConstants.useFilePanelS, IVConstants.trueS);
                     properties.setProperty(PropertiesConstants.cacheThumbsS, IVConstants.trueS);
                     properties.setProperty(PropertiesConstants.createThumbsInBackgroundS, IVConstants.trueS);

        }

	private void createMenus() {

		JMenuBar menuBar = new JMenuBar();

                // create "File" menu
                createFileMenu(menuBar);

                // create "Edit" Menu
                JMenu editMenu = new JMenu("Edit");
                editMenu.setMnemonic(KeyEvent.VK_E);
                JMenuItem prefs = new JMenuItem("Preferences...");
                prefs.setMnemonic(KeyEvent.VK_P);
                prefs.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P, ActionEvent.CTRL_MASK));
                prefs.addActionListener(this);
                prefs.setActionCommand(IVConstants.preferencesS);
                editMenu.add(prefs);
                menuBar.add(editMenu);

                // create "View" Menu
                createViewMenu(menuBar);

                // create "Options" Menu
                if (useFilePanel) {
                   JMenu optionsMenu = new JMenu("Options");
                   optionsMenu.setMnemonic(KeyEvent.VK_T);
                   ocFilePanel = new JMenuItem(IVConstants.cfp);
		   ocFilePanel.setMnemonic(KeyEvent.VK_C);
                   ocFilePanel.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_B, ActionEvent.CTRL_MASK));
                   ocFilePanel.addActionListener(this);
                   ocFilePanel.setActionCommand(IVConstants.ocFilePanelS);
                   optionsMenu.add(ocFilePanel);
                   menuBar.add(optionsMenu);
                }

                // create "Help" Menu
                JMenu helpMenu = new JMenu("Help");
                helpMenu.setMnemonic(KeyEvent.VK_H);
                JMenuItem about = new JMenuItem("About");
                about.setMnemonic(KeyEvent.VK_A);
                about.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A, ActionEvent.CTRL_MASK));
                about.addActionListener(this);
                about.setActionCommand(IVConstants.aboutS);
                helpMenu.add(about);
                menuBar.add( Box.createHorizontalGlue() ); // space help menu to far right
                menuBar.add(helpMenu);

                setJMenuBar(menuBar);

	}

	private void createFileMenu(JMenuBar menuBar) {

		JMenu fileMenu = new JMenu("File");
                fileMenu.setMnemonic(KeyEvent.VK_F);
                JMenuItem open, save, close, quit;
                open = new JMenuItem("Open...");
                open.setMnemonic(KeyEvent.VK_O);
                open.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, ActionEvent.CTRL_MASK));
                open.addActionListener(this);
                open.setActionCommand(IVConstants.openS);
                close = new JMenuItem("Close");
                close.setMnemonic(KeyEvent.VK_C);
                close.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_W, ActionEvent.CTRL_MASK));
                close.addActionListener(this);
                close.setActionCommand(IVConstants.closeS);
		quit = new JMenuItem("Exit");
                quit.setMnemonic(KeyEvent.VK_X);
                quit.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q, ActionEvent.CTRL_MASK));
                quit.addActionListener(this);
                quit.setActionCommand(IVConstants.exitS);
                fileMenu.add(open);
                fileMenu.add(close);
                fileMenu.addSeparator();
                fileMenu.add(quit);
                menuBar.add(fileMenu);

	}

	private void createViewMenu(JMenuBar menuBar) {

		JMenu viewMenu = new JMenu("View");
                viewMenu.setMnemonic(KeyEvent.VK_V);
                JMenuItem scale, fullScreen, max, min;
                scale = new JMenuItem("Scale...");
                scale.setMnemonic(KeyEvent.VK_S);
                scale.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK));
                scale.addActionListener(this);
                scale.setActionCommand(IVConstants.scaleS);
                viewMenu.add(scale);
                JMenuItem zoomIn, zoomOut;
                zoomIn = new JMenuItem("Zoom In");
                zoomIn.setMnemonic(KeyEvent.VK_I);
                zoomIn.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_G, ActionEvent.CTRL_MASK));
                zoomIn.addActionListener(this);
                zoomIn.setActionCommand(IVConstants.zoomInS);
                viewMenu.add(zoomIn);
                zoomOut = new JMenuItem("Zoom Out");
                zoomOut.setMnemonic(KeyEvent.VK_O);
                zoomOut.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_L, ActionEvent.CTRL_MASK));
                zoomOut.addActionListener(this);
                zoomOut.setActionCommand(IVConstants.zoomOutS);
                viewMenu.add(zoomOut);
                viewMenu.addSeparator();
                min = new JMenuItem("Minimize");
                min.setMnemonic(KeyEvent.VK_M);
                min.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_M, ActionEvent.CTRL_MASK));
                min.addActionListener(this);
                min.setActionCommand(IVConstants.minS);
                viewMenu.add(min);
                max = new JMenuItem("Maximize");
                max.setMnemonic(KeyEvent.VK_X);
                max.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X, ActionEvent.CTRL_MASK));
                max.addActionListener(this);
                max.setActionCommand(IVConstants.maxS);
                viewMenu.add(max);
                viewMenu.addSeparator();
                fullScreen = new JMenuItem("Full Screen");
                fullScreen.setMnemonic(KeyEvent.VK_U);
                fullScreen.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F, ActionEvent.CTRL_MASK));
                fullScreen.addActionListener(this);
                fullScreen.setActionCommand(IVConstants.fullScreenS);
                viewMenu.add(fullScreen);
                menuBar.add(viewMenu);

	}

	/**
	 * Create the popup (right-click) menu.
	 */
        private void createPopupMenu() {

                popup = new JPopupMenu();

                JMenuItem scale, zoomIn, zoomOut, min, max, fullScreen;
                scale = new JMenuItem("Scale...");
                scale.setMnemonic(KeyEvent.VK_S);
                scale.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK));
                scale.addActionListener(this);
                scale.setActionCommand(IVConstants.scaleS);
                popup.add(scale);
                zoomIn = new JMenuItem("Zoom In");
                zoomIn.setMnemonic(KeyEvent.VK_I);
                zoomIn.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_G, ActionEvent.CTRL_MASK));
                zoomIn.addActionListener(this);
                zoomIn.setActionCommand(IVConstants.zoomInS);
                popup.add(zoomIn);
                zoomOut = new JMenuItem("Zoom Out");
                zoomOut.setMnemonic(KeyEvent.VK_O);
                zoomOut.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_L, ActionEvent.CTRL_MASK));
                zoomOut.addActionListener(this);
                zoomOut.setActionCommand(IVConstants.zoomOutS);
                popup.add(zoomOut);
                popup.addSeparator();
                min = new JMenuItem("Minimize");
                min.setMnemonic(KeyEvent.VK_M);
                min.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_M, ActionEvent.CTRL_MASK));
                min.addActionListener(this);
                min.setActionCommand(IVConstants.minS);
                popup.add(min);
                max = new JMenuItem("Maximize");
                max.setMnemonic(KeyEvent.VK_X);
                max.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X, ActionEvent.CTRL_MASK));
                max.addActionListener(this);
                max.setActionCommand(IVConstants.maxS);
                popup.add(max);
                popup.addSeparator();
                fullScreen = new JMenuItem("Full Screen");
                fullScreen.setMnemonic(KeyEvent.VK_U);
                fullScreen.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F, ActionEvent.CTRL_MASK));
                fullScreen.addActionListener(this);
                fullScreen.setActionCommand(IVConstants.fullScreenS);
                popup.add(fullScreen);

        }

	private void createToolBar() {

		toolBar = new JToolBar();

		createButton("icons/Open24.gif", "Open", IVConstants.openS);
		toolBar.addSeparator();

		createButton("icons/Zoom24.gif", "Scale/Zoom", IVConstants.scaleS);
		createButton("icons/ZoomIn24.gif", "Zoom In", IVConstants.zoomInS);
		createButton("icons/ZoomOut24.gif", "Zoom Out", IVConstants.zoomOutS);

		toolBar.addSeparator();
		createScaleChoice();
		toolBar.addSeparator();

                createButton("icons/Preferences24.gif", "Preferences", IVConstants.preferencesS);
                toolBar.addSeparator();

                createButton("icons/About24.gif", "About", IVConstants.aboutS);

	}

	private void createButton(String filename, String toolTipText, String actionCommand) {

		URL iconURL = ClassLoader.getSystemResource(filename);
		ImageIcon icon = new ImageIcon(iconURL);
	        JButton button = new JButton(icon);
      	 	button.setToolTipText(toolTipText);
	 	button.addActionListener(this);
	 	button.setActionCommand(actionCommand);
	 	toolBar.add(button);

	}

	/**
	 * Creates the combo box for scaling an image.
	 */
	private void createScaleChoice() {

		scaleSizes = new String[7];
	 	int i;
	 	int j = 0;
	 	for (i = 4; i > 0; i--) { // create items 4x to 1x
	 	    scaleSizes[j] = ( new Integer(i) ).toString() + IVConstants.xS;
	 	    j++;
	 	}
	 	String div = "/";
	 	for (i = 2; i < 5; i++) { // create items 1/2x to 1/4x
	 	    scaleSizes[j] = IVConstants.oneS + div + ( new Integer(i) ).toString() +
	 	                     IVConstants.xS;
	 	    j++;
	 	}
                scaleChoice = new JComboBox(scaleSizes);
                scaleChoice.setEditable(true);
                scaleChoice.setSelectedIndex(3); // set 1x to be the selected value
                scaleChoice.addActionListener(this);
                scaleChoice.setActionCommand(IVConstants.scaleChoiceS);
                scaleChoice.setToolTipText("Scale/Zoom");
                scaleChoice.setMaximumSize( new Dimension(80, 25) );
                //scaleChoice.setAlignmentX(Component.RIGHT_ALIGNMENT);
                toolBar.add(scaleChoice);

	}

	public void actionPerformed(ActionEvent e) {

		String actionCommand = e.getActionCommand();

		if ( actionCommand.equals(IVConstants.openS) )
		   openAction();
		else if ( actionCommand.equals(IVConstants.closeS) )
		   closeAction();
	        else if ( actionCommand.equals(IVConstants.exitS) )
		   exitAction();
		else if ( actionCommand.equals(IVConstants.preferencesS) )
		   prefsAction();
		else if ( actionCommand.equals(IVConstants.scaleS) ) // scale menu item
		   scaleAction();
		else if ( actionCommand.equals(IVConstants.fullScreenS) )
		   fullScreenAction();
	        else if ( actionCommand.equals(IVConstants.aboutS) )
                   aboutAction();
                else if ( actionCommand.equals(IVConstants.scaleChoiceS) ) // scale combo box
                   scaleChoiceAction();
                else if ( actionCommand.equals(IVConstants.zoomInS) )
                   zoomAction(-1);
                else if ( actionCommand.equals(IVConstants.zoomOutS) )
                   zoomAction(1);
                else if ( actionCommand.equals(IVConstants.minS) )
                   minAction();
                else if ( actionCommand.equals(IVConstants.maxS) )
                   maxAction();
                else if ( actionCommand.equals(IVConstants.ocFilePanelS) )
                   ocFilePanelAction();

	}

	public void resetDragMode(String dragMode) {

		desktopPane.putClientProperty("JDesktopPane.dragMode", dragMode);

	}

	public void resetFileChooser() {

	        useSwingFileChooser = prefsDialog.getProp(PropertiesConstants.useSwingFileChooserS);
	        previewImages = prefsDialog.getProp(PropertiesConstants.previewImagesS);
	        boolean cacheThumbs = prefsDialog.getProp(PropertiesConstants.cacheThumbsS);
	        boolean backgroundCache = prefsDialog.getProp(PropertiesConstants.createThumbsInBackgroundS);
	        if (previewImages && fileChooserCreated) {
	           ImagePreview ip = (ImagePreview)fileChooser.getAccessory();
	           ip.enableThumbnailCaching(cacheThumbs);
	           ip.enableBackgroundThumbsCreation(backgroundCache);
	        }
	        fileChooserCreated = false; // need to rebuild file chooser next time it's opened

	        useFilePanel = prefsDialog.getProp(PropertiesConstants.useFilePanelS);

                String prop = properties.getProperty(PropertiesConstants.imageDirS);
                if ( !prop.endsWith(slashS) )
                   prop += slashS;
                if (filePanel != null) {
                   filePanel.setCurrDirItem(prop);
                   filePanel.enablePreviews(previewImages); // enable/disable previews in filePanel
                   ImagePreview ip = filePanel.getImagePreviewer();
                   ip.enableThumbnailCaching(cacheThumbs);
                   filePanel.enableBackgroundThumbsCreation(backgroundCache);
	        }
	        useJavaLF = prefsDialog.getProp(PropertiesConstants.useJavaLFS);
	        useFileFilters = prefsDialog.getProp(PropertiesConstants.useFileFiltersS);
	        useFileTypeIcons = prefsDialog.getProp(PropertiesConstants.useFileTypeIconsS);

	}

	/**
	 * Opens an existing image file.
	 */
	private void openAction() {

		File file;

		if (useSwingFileChooser) {

		   if (!fileChooserCreated)
		      createFileChooser();

		   int returnVal = fileChooser.showOpenDialog(this); // open the modal file chooser

                   if (returnVal == JFileChooser.APPROVE_OPTION) {

                      if (useJavaLF) { // multi-selection only works w/ Java L&F
                         File[] files = getSelectedFiles(fileChooser);
                         for (int i = 0; i < files.length; i++)
                             showImageAI(files[i]); // use JAI for all images to support scaling
                      }
                      else {
                         file = fileChooser.getSelectedFile();
                         showImageAI(file);
                      }

                   }

                } // end if (useSwingFileChooser)
                else { // native file dialog

		   if (!fileChooserCreated) {
                      fileDialog = new FileDialog(this,"Open",FileDialog.LOAD);
                      fileDialog.setDirectory( properties.getProperty(PropertiesConstants.imageDirS) );
                      fileChooserCreated = true;
                   }

		   fileDialog.setFile(Utils.allTypes); // file filter for Win32

                   fileDialog.setVisible(true);
                   String filename;
                   if ( (filename = fileDialog.getFile() ) != null) {
                      file = new File( fileDialog.getDirectory() + filename );
                      showImageAI(file);
		   }
                }

	}

	private void createFileChooser() {

		fileChooser = new JFileChooser();
		String prop = properties.getProperty(PropertiesConstants.useJavaLFS);
	        useJavaLF = Boolean.valueOf(prop).booleanValue();
		if (useJavaLF)
		   fileChooser.setMultiSelectionEnabled(true);
		prop = properties.getProperty(PropertiesConstants.imageDirS);
		fileChooser.setCurrentDirectory( new File( prop ) );
		if (useFileFilters)
		   fileChooser.addChoosableFileFilter( new ImageFilter() );
                if (useFileTypeIcons)
                   fileChooser.setFileView( new ImageFileView() );
                if (previewImages) {
                   ImagePreview ip = new ImagePreview(fileChooser, this);
                   fileChooser.setAccessory(ip);
                   prop = properties.getProperty(PropertiesConstants.cacheThumbsS);
	           boolean cacheThumbs = Boolean.valueOf(prop).booleanValue();
                   ip.enableThumbnailCaching(cacheThumbs);
                   prop = properties.getProperty(PropertiesConstants.createThumbsInBackgroundS);
	           boolean tf = Boolean.valueOf(prop).booleanValue();
                   ip.enableBackgroundThumbsCreation(tf);
                }
                fileChooserCreated = true;

	}

	/**
	 * This is an ugly hack to allow getting multiple files
	 * from a JFileChooser at once.
	 */
	public static File[] getSelectedFiles(JFileChooser chooser) {

        	Container c1 = (Container)chooser.getComponent(3);
		JList list = null;

		while (c1 != null) {
                      Container c = (Container)c1.getComponent(0);
                      if (c instanceof JList) {
                         list = (JList)c;
                         break;
                      }
                      c1 = c;
                }

		Object[] entries = list.getSelectedValues();
                File[] files = new File[entries.length];

		for (int k=0; k<entries.length; k++) {
                    if (entries[k] instanceof File)
                       files[k] = (File)entries[k];
                }

                return files;

        }

	/**
	 * Closes and disposes of the currently selected internal frame.
	 */
	private void closeAction() {

		ImageInternalFrame frameToClose;

		frameToClose = openFrameList.getSelectedFrame();
		if (frameToClose != null) {
		   openFrameCount--;
		   frameToClose.setVisible(false);
		   frameToClose.dispose();
		   frameToClose = null;
		}

	}

	private void exitAction() {

		System.exit(0);

	}

	private void prefsAction() {

		if (!prefsDialogCreated) {
                   prefsDialog = new PrefsDialog(this, properties);
                   prefsDialogCreated = true;
		}

		prefsDialog.setVisible(true);

	}

	private void scaleAction() {

		if (!scaleDialogCreated) {
		   scaleDialog = new ScaleDialog(this, scaleSizes);
		   scaleDialogCreated = true;
		}

		scaleDialog.setVisible(true);

	}

	private void fullScreenAction() {

		final JWindow window = new JWindow();
		final Container contentPane;

		final ImageInternalFrame selectedFrame = openFrameList.getSelectedFrame();

		if (selectedFrame != null) {
		   contentPane = selectedFrame.getContentPane();
		   window.setContentPane(contentPane);
		   Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
		   window.setSize( (int)d.getWidth(), (int)d.getHeight() );
		   window.addMouseListener(
		   	new MouseAdapter() {
		   		public void mousePressed(MouseEvent e) {
                                             window.setVisible(false);
                                             selectedFrame.setContentPane(contentPane);
                                             selectedFrame.revalidate();
                                             window.dispose();
                                }
                        }
                   );
                   window.addKeyListener(
                   	new KeyAdapter() {
                        	public void keyPressed(KeyEvent e) {
                                	int keyCode =  e.getKeyCode();
                                        if (keyCode == KeyEvent.VK_ESCAPE)
                                           window.setVisible(false);
                                           selectedFrame.setContentPane(contentPane);
                                           selectedFrame.revalidate();
                                           window.dispose();
                                }
                   	}
                   );
		   window.setVisible(true);
		   window.requestFocus();
		}

	}

        /**
         * Opens the Help->About dialog.
         */
        private void aboutAction() {

        	JIVAboutDialog aboutDialog = new JIVAboutDialog(this);

                // center the dialog over the main window
                int width = 600;
                int height = 570;
                aboutDialog.setSize(width, height);
                Point p = getLocationOnScreen();
		Dimension d = getSize();
                Rectangle rect = new Rectangle(new Point(0,0), d);
                aboutDialog.setLocation( (rect.x + ((rect.width - width)/2)),
		               (rect.y + ((rect.height - height)/2)) );
                aboutDialog.setVisible(true);

        }

        private void scaleChoiceAction() {

        	ImageInternalFrame selectedFrame = openFrameList.getSelectedFrame();

		if (selectedFrame != null) {

                   // get the selected item in the scaleChoice (combo box)
                   String scaleString = (String)scaleChoice.getSelectedItem();

                   scale(scaleString, selectedFrame);

                }

	}

	/**
	 * Zoom in or out on the selected frame. <BR>
	 * inOut == -1 if zooming in, 1 if zooming out
	 */
	private void zoomAction(int inOut) {

        	ImageInternalFrame selectedFrame = openFrameList.getSelectedFrame();

		String scaleString;

		if (selectedFrame != null) {

                   scaleString = selectedFrame.getScaleFactor();
                   if ( scaleString.endsWith(IVConstants.xS) ) // remove the "x" from the end
                      scaleString = scaleString.substring(0, scaleString.length() - 1);

                   int index = scaleString.indexOf('/'); // check to see if the selected item is a fraction

                   float scale;
                   try {

                        if (index == -1) // integer
                           scale = Float.parseFloat(scaleString);
                        else { // fraction (ie 1/2)
                           float n = Float.parseFloat( scaleString.substring(0,1) ); // the numerator
                           float d = Float.parseFloat( scaleString.substring(2) ); // the denominator
                           scale = n/d;
                        }
                        if (inOut == -1)
                           scale *= 2F; // zoom in (2x)
                        else
                           scale *= .5F; // zoom out (.5x)

			scaleString = Float.toString(scale) + IVConstants.xS;

                        resize(scale);
		   }
		   catch (NumberFormatException nfe) {}

		   selectedFrame.setScaleFactor(scaleString);
                   scaleChoice.getEditor().setItem(scaleString);

                }

	}

	private void minAction() {

		ImageInternalFrame selectedFrame = openFrameList.getSelectedFrame();
		try {
	             if (selectedFrame != null)
	                selectedFrame.setIcon( !selectedFrame.isIcon() );
	        }
	        catch (PropertyVetoException pve) {}

	}

	private void maxAction() {

		ImageInternalFrame selectedFrame = openFrameList.getSelectedFrame();
		try {
	             if (selectedFrame != null)
		        selectedFrame.setMaximum( !selectedFrame.isMaximum() );
                }
                catch (PropertyVetoException pve) {}

	}

	/**
	 * Opens or closes the file panel by expanding/collapsing the split pane
	 */
	private void ocFilePanelAction() {

		updateSplitPane(true);

	}

	/**
	 * Property change listener for split pane divider location change
	 */
	public void propertyChange(PropertyChangeEvent evt) {

		updateSplitPane(false);

	}

	private void updateSplitPane(boolean resize) {

		int divLoc = splitPane.getDividerLocation();
		String text;
		char key;

		if (divLoc > 10) {
		   if (resize) {
		      splitPane.setDividerLocation(0.0); // close the file panel
		      text = IVConstants.ofp;
		      key = KeyEvent.VK_O;
		   }
		   else {
		      text = IVConstants.cfp;
		      key = KeyEvent.VK_C;
		   }
		   ocFilePanel.setText(text);
		   ocFilePanel.setMnemonic(key);
		}
		else {
		   int lastLoc;
		   if (resize) {
		      lastLoc = splitPane.getLastDividerLocation();
		      splitPane.setDividerLocation(lastLoc); // open the file panel
		      text = IVConstants.cfp;
		      key = KeyEvent.VK_C;
		   }
		   else {
		      text = IVConstants.ofp;
		      key = KeyEvent.VK_O;
		   }
		   ocFilePanel.setText(text);
		   ocFilePanel.setMnemonic(key);
		}

	}

	public ImageInternalFrame getSelectedFrame() {

		return openFrameList.getSelectedFrame();

	}

	public JFileChooser getFileChooser() {

		return fileChooser;

	}

	private void resize(float scaleFactor) {

		ImageInternalFrame selectedFrame = (ImageInternalFrame)openFrameList.getSelectedFrame();
	        scaleActual(selectedFrame, scaleFactor);

	}

	public void scale(String scaleString, ImageInternalFrame selectedFrame) {

		String backupSS = scaleString;

		if ( scaleString.endsWith(IVConstants.xS) ) {
		   // remove the "x" from the end
                   scaleString = scaleString.substring(0, scaleString.length() - 1);
                }

                // check to see if the selected item is a fraction
                int index = scaleString.indexOf('/');

                float scale;
                try {

                    if (index == -1) { // integer
                       scale = Float.parseFloat(scaleString);
                    }
                    else { // fraction (ie 1/2)
                       float n, d;
                       n = Float.parseFloat( scaleString.substring(0,1) ); // the numerator
                       d = Float.parseFloat( scaleString.substring(2) ); // the denominator
                       scale = n/d;
                    }

		    resize(scale);

                }
                catch (NumberFormatException nfe) {
                      JOptionPane.showMessageDialog(this, "\""+scaleString+"\" is not a valid scale factor.",
		                                     "Error", JOptionPane.ERROR_MESSAGE);
                }

		selectedFrame.setScaleFactor(backupSS);
                scaleChoice.getEditor().setItem(backupSS);

	}

        /**
         * Performs the actual image scaling.
         */
        private void scaleActual(ImageInternalFrame selectedFrame, float scaleFactor) {

		DisplayJAI imagePanel = selectedFrame.getImagePanel();
		imagePanel.scale(scaleFactor);

        }

	/**
	 * Opens a JInternalFrame with the image specified
	 * by the File "file".  Called for all images.
	 */
        public void showImageAI(File file) {

                //try {
		     // load the image
		     Image img = Toolkit.getDefaultToolkit().getImage( file.getAbsolutePath() );
                     int width = img.getWidth(this);
                     int height = img.getHeight(this);

                     BufferedImage bufferedImage = new BufferedImage(width, height,
                                                                      BufferedImage.TYPE_INT_RGB);
                     Graphics2D biContext = bufferedImage.createGraphics();
                     biContext.drawImage(img, 0, 0, null);

        	     // add the image to the scroll pane
        	     DisplayJAI imagePanel = new DisplayJAI(bufferedImage);
        	     JScrollPane scrollPane = new JScrollPane(imagePanel);
        	     imagePanel.setScrollPane(scrollPane);

                     addToFrame(scrollPane, file);
                     frame.setImage(bufferedImage); // set the frame's image
                     frame.setImagePanel(imagePanel);

                     width = bufferedImage.getWidth() + 40;
                     height = bufferedImage.getHeight() + 40;

                     openFrame(width, height);
                /*}
                catch (Exception e) {
                    JOptionPane.showMessageDialog(this, "Could Not Open Image.",
	                                          "Error", JOptionPane.ERROR_MESSAGE);
                } */

        }

        // empty mouse methods must be here so that this
        //  class meets the mouse listener interface

        public void mouseClicked(MouseEvent me) {}

        public void mouseEntered(MouseEvent me) {}

        public void mouseExited(MouseEvent me) {}

        public void mousePressed(MouseEvent me) {

		if (me.isPopupTrigger())
                   popup.show(me.getComponent(), me.getX(), me.getY());

	}

        /**
         * Invoked when a mouse button is released - used
         * to listen for mouse clicks on the images,
         * so that the resize window can be opened.
         */
        public void mouseReleased(MouseEvent me) {

		if (me.isPopupTrigger())
                   popup.show(me.getComponent(), me.getX(), me.getY());

        }

        /**
         * Increments openFrameCount, constructs the internal frame,
         * sets its content pane, packs it, and ands an internal
         * frame listener to it.
         */
        private synchronized void addToFrame(JScrollPane scrollPane, File file) {

		frame = new ImageInternalFrame( file.getName(),        // title
                                         	  true,                // resizable
                                                   true,               // closable
                                                    true,              // maximizable
                                                     true,             // iconifiable
                                                      numFramesCreated,
                                                       this );
                numFramesCreated++;
                openFrameList.addElement(frame);
                frame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
                frame.setContentPane(scrollPane);
                frame.addMouseListener(this);
                frame.pack();
                openFrameCount++;

        }

        private void openFrame(int width, int height) {

		setIFrameSize(width, height, frame);

                // tile the internal frames
                int xLoc, yLoc;
                xLoc = IVConstants.xOffset * ( openFrameCount % 10 );
                yLoc = IVConstants.yOffset * ( openFrameCount % 10 );
                frame.setLocation(xLoc, yLoc);
                frame.setVisible(true); //necessary as of kestrel
                desktopPane.add(frame);
                try {
                  frame.setSelected(true);
                }
                catch (java.beans.PropertyVetoException e) {}

        }

	private void setIFrameSize(int width, int height, ImageInternalFrame iFrame) {

		if ( (width < IVConstants.minWidth) ||
		       (height < IVConstants.minHeight) ) // below min height or width
                   iFrame.setSize( IVConstants.minWidth, IVConstants.minHeight );
                else if ( (width < IVConstants.maxWidth) && (height < IVConstants.maxHeight) )
                   iFrame.setSize( width, height );
                else if ( (width >= IVConstants.maxWidth) &&
                            (height >= IVConstants.maxHeight) ) // exceeds max width and height
                   iFrame.setSize( IVConstants.maxWidth, IVConstants.maxHeight );
                else if (width >= IVConstants.maxWidth) // exceeds max width
                   iFrame.setSize( IVConstants.maxWidth, height );
                else if (height >= IVConstants.maxHeight) // exceeds max height
                   iFrame.setSize( width, IVConstants.maxHeight );

	}

        public void decrementOpenFrameCount() {

        	openFrameCount--;

        }

        public void updateSelectedFrame(int index, String scaleFactor) {

        	openFrameList.setSelectedFrame(index);

        	// set the scale factor displayed in the combo box to scaleFactor
        	scaleChoice.getEditor().setItem(scaleFactor);

        }

	public static void main(String[] args) {

                ImageViewer imageViewer = new ImageViewer(args);
		imageViewer.addWindowListener(
			     new WindowAdapter() {
				public void windowClosing(WindowEvent e) {
				   System.exit(0);
				}
		             }
		);

	}

}
